/*
 Mini Temp Logger - Data Transfer
 Version: v1.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 22/02/2015
 Last Modified: 22/03/2017
   
 Used to interface to the Mini Temp Logger and extract the logged data.
 
 Based on V-USB example code by Christian Starkjohann and AVR ATtiny USB Tutorial at http://codeandlife.com
 
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "setup.c"

int main(int argc, char **argv) {
	
	printf("Mini Temp Logger - Data Transfer v1.0 by insideGadgets\n"\
			 "------------------------------------------------------\n"\
			 "Data transfer interface - Used to transfer the logged data\n\n");
	
	handle = usbOpenDevice(0x16C0, "insidegadgets.com", 0x05DC, "MTL");
	if (handle == NULL) {
		fprintf(stderr, "Could not find USB device!\n");
		exit(1);
	}
	
	int nBytes = 0;
	
	// Get the date/time to be used for the filename
	time_t rawtime;
	struct tm* timeinfo;
	char timebuffer[80];
	
	time (&rawtime);
	timeinfo = localtime (&rawtime);
	
	strftime (timebuffer,80,"%Y-%m-%d_%H-%M-%S.txt",timeinfo);
	
	// Create a new file
	FILE *pFile = fopen(timebuffer, "wb");

	
	// Get RTC time
	nBytes = usb_control_msg(handle, 
			USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, 
			USB_RTC_START_LOG_TIME, 0, 0, (char *)buffer, sizeof(buffer), 5000);
	
	if (nBytes == 0) {
		printf("Error: Failed to receive data when requesting RTC time\n");
	}
	
	uint8_t readRtcAmPm = buffer[0];
	uint8_t readRtcSeconds = buffer[1];
	uint8_t readRtcMinutes = buffer[2];
	uint8_t readRtcHour = buffer[3];
	uint8_t readRtcDate = buffer[4];
	uint8_t readRtcMonth = buffer[5];
	uint8_t readRtcYear = buffer[6];
	
	time_t rtcUnixTime;
	struct tm* rtcUnixTm = 0;
	time (&rtcUnixTime);
	rtcUnixTm = localtime(&rtcUnixTime);
	
	// Convert RTC stored start time to Unix timestamp
	rtcUnixTm->tm_sec = readRtcSeconds;
	rtcUnixTm->tm_min = readRtcMinutes;
	rtcUnixTm->tm_hour = readRtcHour;
	
	if (readRtcAmPm == 1 && readRtcHour <= 11) {
		rtcUnixTm->tm_hour += 12;
	}
	else if (readRtcAmPm == 0 && readRtcHour == 12) {
		rtcUnixTm->tm_hour = 0;
	}
	
	rtcUnixTm->tm_mday = readRtcDate;
	rtcUnixTm->tm_mon = readRtcMonth - 1;
	rtcUnixTm->tm_year = readRtcYear + 100;
	
	rtcUnixTime = mktime(rtcUnixTm);
	
	printf("RTC Start time is 20%i year, %i month, %i date, %i hour, %i minutes, %i seconds ", readRtcYear, readRtcMonth, readRtcDate, readRtcHour, readRtcMinutes, readRtcSeconds);
	if (readRtcAmPm == 0) {
		printf("AM\n");
	}
	else {
		printf("PM\n");
	}
	
	Sleep(50);
	
	// Get function in use
	nBytes = usb_control_msg(handle, 
				USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, 
				USB_FUNCTION_SELECTED, 0, 0, (char *)buffer, sizeof(buffer), 5000);
	
	uint8_t readFunctionSelected = buffer[0];
	
	// Get delay time
	nBytes = usb_control_msg(handle, 
			USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, 
			USB_DELAY_TIME, 0, 0, (char *)buffer, sizeof(buffer), 5000);
	
	uint8_t delayTimeSeconds = buffer[0];
	uint8_t delayTimeMinutes = buffer[1];
	uint8_t delayTimeHours = buffer[2];
	uint8_t delayTimeDays = buffer[3];
	
	// Calculate delay time
	uint32_t delayCountInSeconds = delayTimeSeconds + (delayTimeMinutes * 60) + (delayTimeHours * 3600) + (delayTimeDays * 86400);
	
	// Print delay time
	char tempbuffer[50];
	char delayString[100];
	strncpy(delayString, "Delay Time: ", 13);
	
	if (delayTimeDays > 0) {
		sprintf(tempbuffer, "%d Days", delayTimeDays);
		strcat(delayString, tempbuffer);
	}
	if (delayTimeHours > 0) {
		if (delayTimeDays > 0) {
			strcat(delayString, ", ");
		}
		sprintf(tempbuffer, "%d Hours", delayTimeHours);
		strcat(delayString, tempbuffer);
	}
	if (delayTimeMinutes > 0) {
		if (delayTimeDays > 0 || delayTimeHours > 0) {
			strcat(delayString, ", ");
		}
		sprintf(tempbuffer, "%d Minutes", delayTimeMinutes);
		strcat(delayString, tempbuffer);
	}
	if (delayTimeSeconds > 0) {
		if (delayTimeDays > 0 || delayTimeHours > 0 || delayTimeMinutes > 0) {
			strcat(delayString, ", ");
		}
		sprintf(tempbuffer, "%d Seconds", delayTimeSeconds);
		strcat(delayString, tempbuffer);
	}
	
	strcat(delayString, "\r\n");
	printf("%s", delayString);
	fwrite(delayString, 1, strlen(delayString), pFile);
	
	
	// Transfer data
	printf("Transferring data...\n");
	uint32_t readCount = 0;
	uint8_t inTransfer = 1;
	nBytes = 1;
	while (inTransfer == 1) { 
		nBytes = usb_control_msg(handle, 
			USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, 
			USB_TRANSFER, 0, 0, (char *)buffer, sizeof(buffer), 5000);
		
		if (nBytes > 0) {
			uint8_t highByte = 0;
			uint8_t lowByte = 0;
			uint8_t charCount = 0;
			nBytes -= 2;
			
			while (nBytes >= 0) {
				if (charCount % 2 == 0) {
					highByte = buffer[charCount];
				}
				else {
					lowByte = buffer[charCount];
					int combinedValue = (highByte << 8) | lowByte;
					
					readCount += 2;
					
					// End if we reach the the end logging number or if we've read all of the EERPROM
					if (combinedValue == endLoggingNumber || readCount >= 131071) {
						inTransfer = 0;
						break;
					}
					
					// Convert new time
					rtcUnixTm = localtime (&rtcUnixTime);
					char tempTimeBuffer[80];
					strftime (tempTimeBuffer, 80, "%d/%m/%Y %I:%M:%S %p", rtcUnixTm);
					fwrite(tempTimeBuffer, 1, strlen(tempTimeBuffer), pFile);
					printf("%s  ", tempTimeBuffer);
					
					char tempReadingBuffer[60];
					if (readFunctionSelected == LOG_TEMPERATURE) { 
						// Convert logged reading, 0 to 150C (0x000 to 0x960), -55 to -0.06C (1C90 to 1FFF)
						if (combinedValue >= 0x1C90 && combinedValue <= 0x1FFF) { // Convert negative temps
							combinedValue &= 0x1FF;
							combinedValue--;
							combinedValue ^= 0x1FF;
							sprintf(tempReadingBuffer, "\t-%4.2f\r\n", combinedValue * 0.0625);
							printf("-");
						}
						else {
							sprintf(tempReadingBuffer, "\t%4.2f\r\n", combinedValue * 0.0625);
						}
						fwrite(tempReadingBuffer, 1, strlen(tempReadingBuffer), pFile);
						printf("%4.2f\n", combinedValue * 0.0625);
					}
					else {
						// Convert logged reading to voltage 
						sprintf(tempReadingBuffer, "\t%2.3f\r\n", (double) combinedValue * (double) (0.00107421875 * 21)  - (double) (combinedValue * 0.00021484375)); // 0.000214 is the calculated difference between the actual ADC result and what the device really measures
						fwrite(tempReadingBuffer, 1, strlen(tempReadingBuffer), pFile);
						printf("%2.3f\n", (double) combinedValue * (double) (0.00107421875 * 21) - (double) (combinedValue * 0.00021484375));
					}
					
					// Increment time by delay time
					rtcUnixTime += delayCountInSeconds;
				}
				
				charCount++;
				nBytes--;
			}
		}
		else {
			inTransfer = 0;
		}
	}
	
	printf("Transfer complete\n");
	usb_close(handle);
	fclose(pFile);
	
	return 0;
}
